//
//  MyDocument.m
//  MVV
//
//  Created by Eivind Danielsen on 10.03.09.
//  Copyright NTNU 2009 . All rights reserved.
//

#import "MyDocument.h"

@implementation MyDocument 

//init variables and allocate memory for mutable arrays
- (id)init
{
    self = [super init];
    if (self) {
		pathArray = [[NSMutableArray alloc]init];
		pathArrayMovieViewIndex = [[NSMutableArray alloc]init];
		qtMovieArray = [[NSMutableArray alloc]init];
		playing = 0;
		pause = FALSE;
		initPlay = FALSE;
		}
	
    return self;
}

//getter for drawer
- (id)drawer {
	return mDrawer;
}

- (NSString *)windowNibName
{
    // Override returning the nib file name of the document
    // If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
    return @"MyDocument";
}

//Run after the controller runs the nib
- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
	movieViewsArray = [[NSArray arrayWithObjects:movieView1,movieView2, movieView3, movieView4,movieView5, nil]retain];
	
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	for (int i = 0; i < [movieViewsArray count]; i++) {
		[[movieViewsArray objectAtIndex:i] setControllerVisible:NO];
	}
	[pool release];
	
	[mVolumeSlider setMaxValue:1.5];
	
    [super windowControllerDidLoadNib:aController];
	
}

//Get the number of playlist items determined by the pathArray array.
- (int)numberOfRowsInTableView:(NSTableView *)tableView {
	return [pathArray count];
}

//Get the playlist object at tablecolumn and row
- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row {
	
	id value;
	
	if ([[tableColumn identifier] isEqualToString:@"movieview"]) {
		value = [pathArrayMovieViewIndex objectAtIndex:row];
	}
	
	else if ([[tableColumn identifier] isEqualToString:@"file"]){
		value = [pathArray objectAtIndex:row];
	}
	return value;
}

//return the current time of the movie played in the active view.
- (QTTime)getCurrentTime {
	return  [[(QTMovieView *)[movieViewsArray objectAtIndex:0] movie] currentTime];
}

//returns the i'th movieview
- (QTMovieView *)getMovieView:(int)i {
	return [movieViewsArray objectAtIndex:i];
}

//Manage the open file dialog. Stores filepaths in patharray
- (IBAction)openFilesToArray:(id)sender {
	int i; 
	// Create the File Open Dialog class.
	NSOpenPanel* openDlg = [NSOpenPanel openPanel];
	
	// Enable the selection of files in the dialog.
	[openDlg setCanChooseFiles:YES];
	[openDlg setAllowsMultipleSelection:YES];
	
	// Enable the selection of directories in the dialog.
	[openDlg setCanChooseDirectories:NO];
	
	// Display the dialog.  If the OK button was pressed,
	// process the files.
	if ( [openDlg runModalForDirectory:nil file:nil] == NSOKButton )
	{
		// Get an array containing the full filenames of all
		// files and directories selected.
		NSArray* files = [openDlg filenames];
		// Loop through all the files and process them.
		for( i = 0; i < [files count]; i++ )
		{
			NSString* fileName = [files objectAtIndex:i];
			[pathArray addObject:fileName];
			[pathArrayMovieViewIndex addObject:[NSNumber numberWithInt:[pathArrayMovieViewIndex count]+1]];
		}
	}
	[mTableView reloadData];
	initPlay = FALSE;
	[self playVideo:nil];
	
}

//Manages initialplay and play/pause functionality.
- (IBAction)playVideo:(id)sender 
{
	if (initPlay == FALSE) {
		
		for (int i = playing; i < [pathArray count] ; i++)
		{
			if (playing >= [movieViewsArray count]) break;
			
			@try {
				[qtMovieArray addObject:[[QTMovie alloc] initWithFile:[pathArray objectAtIndex:i] error:nil]];
				[[movieViewsArray objectAtIndex:i] setMovie:[qtMovieArray objectAtIndex:i]];
				if (i == 0) [[qtMovieArray objectAtIndex:i] setMuted:FALSE];
				else [[qtMovieArray objectAtIndex:i] setMuted:TRUE];
				playing++;
			}
			@catch (NSException *e) {
				[[qtMovieArray objectAtIndex:i] release];
			}
		}
		initPlay = TRUE;
		pause = FALSE;
	}
	
	if ((QTMovie *)[qtMovieArray objectAtIndex:0]) {
		[mPositionSlider setMaxValue:[[(QTMovieView *)[movieViewsArray objectAtIndex:0] movie] duration].timeValue];
		timeScale = [[[(QTMovieView *)[movieViewsArray objectAtIndex:0] movie] attributeForKey:QTMovieTimeScaleAttribute] longValue];
		timer = [[NSTimer timerWithTimeInterval:0.1 target:self selector:@selector(updateSlider:) userInfo:nil repeats:YES] retain];
		[[NSRunLoop currentRunLoop] addTimer:timer forMode:(NSString *)kCFRunLoopCommonModes];
	}
	
	
	for (int i = 0 ; i < [qtMovieArray count]; i++) {
		[(QTMovie *)[qtMovieArray objectAtIndex:i] setCurrentTime:[self getCurrentTime]];
	}
	
	//[self setMasterTime];
	
	if (pause == FALSE) {
		
		for (int i = 0; i < [qtMovieArray count] ; i++) {
			[(QTMovie *)[qtMovieArray objectAtIndex:i] play];
		}
		pause = TRUE;
	}
	
	else if (pause == TRUE) {
		for (int i = 0; i < [qtMovieArray count] ; i++) {
			[[movieViewsArray objectAtIndex:i] pause:[qtMovieArray objectAtIndex:i]];
		}
		pause = FALSE;
	}
}

//Sets the time of the movie according to slider position.
- (IBAction)setTrackPosition:(id)sender {
	
	QTTime mTime;

	mTime.timeValue = [sender doubleValue];	
	mTime.timeScale = timeScale;
	
	for (int i = 0 ; i < [qtMovieArray count]; i++) {
		[(QTMovie *)[qtMovieArray objectAtIndex:i] setCurrentTime:mTime];
	}
	//[self setMasterTime];
}

- (IBAction)setVolume:(id)sender {
	float mVolume;
	mVolume = [sender floatValue];
	for (int i = 0 ; i < [qtMovieArray count]; i++) {
		[[(QTMovieView *)[movieViewsArray objectAtIndex:i] movie] setVolume:mVolume];
	}
}

//Updates the positionslider value
- (void)updateSlider:(id)timer {
	[mPositionSlider setDoubleValue:[[(QTMovieView *)[movieViewsArray objectAtIndex:0] movie] currentTime].timeValue];
}

//Switches the view according an int value. 
- (void)switchView:(int)i {
	
	switch (i) {
		case 0:
			break;
		case 1:
			[qtMovieArray exchangeObjectAtIndex:0 withObjectAtIndex:i];
			[[qtMovieArray objectAtIndex:0] setMuted:FALSE];
			[[qtMovieArray objectAtIndex:i] setMuted:TRUE];
			[pathArray exchangeObjectAtIndex:0 withObjectAtIndex:i];
			[[movieViewsArray objectAtIndex:0] setMovie:[qtMovieArray objectAtIndex:0]];
			[[movieViewsArray objectAtIndex:i] setMovie:[qtMovieArray objectAtIndex:i]];
			break;
		case 2:
			[qtMovieArray exchangeObjectAtIndex:0 withObjectAtIndex:i];
			[[qtMovieArray objectAtIndex:0] setMuted:FALSE];
			[[qtMovieArray objectAtIndex:i] setMuted:TRUE];
			[pathArray exchangeObjectAtIndex:0 withObjectAtIndex:i];
			[[movieViewsArray objectAtIndex:0] setMovie:[qtMovieArray objectAtIndex:0]];
			[[movieViewsArray objectAtIndex:i] setMovie:[qtMovieArray objectAtIndex:i]];
			break;
		case 3:
			[qtMovieArray exchangeObjectAtIndex:0 withObjectAtIndex:i];
			[[qtMovieArray objectAtIndex:0] setMuted:FALSE];
			[[qtMovieArray objectAtIndex:i] setMuted:TRUE];
			[pathArray exchangeObjectAtIndex:0 withObjectAtIndex:i];
			[[movieViewsArray objectAtIndex:0] setMovie:[qtMovieArray objectAtIndex:0]];
			[[movieViewsArray objectAtIndex:i] setMovie:[qtMovieArray objectAtIndex:i]];
			break;
		case 4:
			[qtMovieArray exchangeObjectAtIndex:0 withObjectAtIndex:i];
			[[qtMovieArray objectAtIndex:0] setMuted:FALSE];
			[[qtMovieArray objectAtIndex:i] setMuted:TRUE];
			[pathArray exchangeObjectAtIndex:0 withObjectAtIndex:i];
			[[movieViewsArray objectAtIndex:0] setMovie:[qtMovieArray objectAtIndex:0]];
			[[movieViewsArray objectAtIndex:i] setMovie:[qtMovieArray objectAtIndex:i]];
			break;
		default:
			break;
	}

	for (int i = 0 ; i < [qtMovieArray count]; i++) {
		[(QTMovie *)[qtMovieArray objectAtIndex:i] setCurrentTime:[self getCurrentTime]];
	}
	//[self setMasterTime];
	
	for (int i = 0 ; i < [qtMovieArray count]; i++) {
		[(QTMovie *)[qtMovieArray objectAtIndex:i] play];
	}
	
	[mTableView reloadData];
}

//Receives mouseevents and determines which view is clicked on.
- (void)handleMovieViewClick:(QTMovieView *)sender {
	for (int i = 1; i < [movieViewsArray count]; i++) {
		if (sender == [movieViewsArray objectAtIndex:i]) {
			[self switchView:i];
		}
	}
}

//Action to open and close the drawer.
- (IBAction)toggleDrawer:(id)sender {
	[mDrawer toggle:sender];
}

//Opens and closes the playlistpanel and reload the data.
- (IBAction)PlaylistPanelOpen:(id)sender {
	
	if ([PlaylistPanel isVisible]) [PlaylistPanel close];
	else [PlaylistPanel makeKeyAndOrderFront:self];
	[mTableView reloadData];
}

//sets movieview 1 as the master time. The other views are slave to this one.
//NOTE: Does not work with the slider. Makes it skip.
- (void) setMasterTime {
	TimeBase mtb = GetMovieTimeBase([[(QTMovieView *)[movieViewsArray objectAtIndex:0] movie] quickTimeMovie]);
	for (int i = 1; i < [qtMovieArray count]; i++) {
			SetTimeBaseMasterTimeBase(mtb, GetMovieTimeBase([[(QTMovieView *)[movieViewsArray objectAtIndex:i] movie] quickTimeMovie]), nil); 
	}	
}

//The subviews can be switched by dragging one into another. The sender is of type
//QTMovieView and the point is in the other view where mouseup is called.
- (void)updateMovieView:(QTMovieView *) sender andPoint:(NSPoint)p{
	
	NSRect rect1;
	NSRect rect2;
	int index1;
	int index2;
	
	if ( sender == [movieViewsArray objectAtIndex:0] ) return;
	
	for (int i = 1 ; i < [movieViewsArray count]; i++) {
		
		if ( sender == [movieViewsArray objectAtIndex:i] ) {
			rect1 = [[movieViewsArray objectAtIndex:i] frame];
			index1 = i;
		}
		if (p.y < [[movieViewsArray objectAtIndex:i] frame].origin.y + [[movieViewsArray objectAtIndex:i] frame].size.height & p.y > [[movieViewsArray objectAtIndex:i] frame].origin.y
			& p.x < [[movieViewsArray objectAtIndex:i] frame].origin.x + [[movieViewsArray objectAtIndex:i] frame].size.width & p.x > [[movieViewsArray objectAtIndex:i] frame].origin.x) {
			rect2 = [[movieViewsArray objectAtIndex:i] frame];
			index2 = i;
		}
	}
	[[movieViewsArray objectAtIndex:index2] setFrame:rect1];
	[[movieViewsArray objectAtIndex:index1] setFrame:rect2];

}

- (void)openPopUpMenu:(id)sender {
	
}

//deallocate memory
- (void)dealloc {
	[movieView1 dealloc];
	[movieView2 dealloc];
	[movieView3 dealloc];
	[movieView4 dealloc];
	[movieView5 dealloc];
	[pathArray release];
	[pathArrayMovieViewIndex release];
	[qtMovieArray release];
	[movieViewsArray release];
	[super dealloc];
}

@end
